@@ -6,8 +6,9 @@ from . import physics
 from . import precompute
 from . import stacking
 from . import surveys
+import importlib.metadata
+
 
 __all__ = ["helpers", "jackknife", "physics", "precompute", "stacking",
            "surveys"]
-
-__version__ = "0.6.1"
+__version__ = importlib.metadata.version("dsigma")
